@@ -5,11 +5,12 @@
 # Distributed under a BSD-like license. For full terms see the file LICENSE.txt
 #
 
+import ConfigParser
+import datetime
 import os
 import sys
 import time
-import datetime
-import ConfigParser
+import traceback
 from functools import wraps
 
 import git
@@ -292,10 +293,68 @@ Git config '%s' missing, please fill it in by executing
     @requires_repo
     def require_remote(self, name):
         try:
-            return self.repo.remotes[name]
+            remote = self.repo.remotes[name]
         except IndexError:
             raise NoSuchRemoteError("Remote '{0}' was not found.".format(name))
 
+        # Monkey patch remote.fetch to fix a false negative assertion in GitPython.
+        class RemoteWrapper(object):
+            def __init__(self, remote):
+                self._remote = remote
+
+            def __str__(self):
+                return self._remote.__str__()
+
+            def __repr__(self):
+                return self._remote.__repr__()
+
+            def __eq__(self, other):
+                return self._remote.__eq__(other)
+
+            def __ne__(self, other):
+                return self._remote.__ne__(other)
+
+            def __hash__(self):
+                return self._remote.__hash__()
+
+            # For some reason __getattribute__ is making infinite recursion,
+            # so let's use __getattr__.
+            def __getattr__(self, name):
+                return self._remote.__getattribute__(name)
+
+            # This is the main monkey patching part.
+            def fetch(self, refspec=None, progress=None, **kwargs):
+                # Let's try 3 times...
+                err = None
+                for x in range(3):
+                    try:
+                        return self._remote.fetch(refspec=refspec,
+                                progress=progress, **kwargs)
+                    except AssertionError as e:
+                        err = e
+                        # If we are in '_get_fetch_info_from_stderr',
+                        # it's the broken assertion and we skip it.
+                        func_name = traceback.extract_stack()[-1][2]
+                        if func_name != '_get_fetch_info_from_stderr':
+                            raise e
+                        err = None
+                # If we somehow get the same exception 3 times, just raise anyway.
+                raise err
+
+        return RemoteWrapper(remote)
+
+    @requires_repo
+    def require_origin_branch(self, branch_name):
+        origin_name = self.origin_name(branch_name)
+        refs = [r
+                for r in self.repo.refs
+                if isinstance(r, RemoteReference) and r.name == origin_name]
+        if len(refs) != 0:
+            assert len(refs) == 1
+            return refs[0]
+        raise NoSuchBranchError(
+                "Remote branch '{0}' was not found".format(origin_name))
+
     def origin(self):
         return self.require_remote(self.origin_name())
 
@@ -452,11 +511,9 @@ Git config '%s' missing, please fill it in by executing
             for b in self.git.branch('-a', '--contains', commit).splitlines()]
 
 
-    def must_be_uptodate(self, branch, fetch):
+    def must_be_uptodate(self, branch):
         remote_branch = self.origin_name(branch)
         if remote_branch in self.branch_names(remote=True):
-            if fetch:
-                self.origin().fetch(branch)
             self.require_branches_equal(branch, remote_branch)
 
     @requires_repo
@@ -476,7 +533,7 @@ Git config '%s' missing, please fill it in by executing
             commit1 = self.repo.rev_parse(branch1)
             commit2 = self.repo.rev_parse(branch2)
         except git.BadObject, e:
-            raise NoSuchBranchError(e.args[0])
+            raise NoSuchBranchError('Branch {0} not found'.format(e.args[0]))
         if commit1 == commit2:
             return 0
         try:
@@ -505,7 +562,7 @@ Git config '%s' missing, please fill it in by executing
                 # Warn here, since there is no harm in being ahead
                 warn("And local branch '%s' is ahead of '%s'." % (branch1, branch2))
             else:
-                raise SystemExit("Branches need merging first.")
+                raise SystemExit("Operation canceled, branches need merging first.")
 
     @requires_repo
     def start_transaction(self, message=None):
@@ -668,14 +725,24 @@ Git config '%s' missing, please fill it in by executing
             The checked out :class:`git.refs.Head` branch.
         """
         mgr = self.managers[identifier]
+        err = None
         try:
             branch = mgr.by_name_prefix(name)
             return branch.checkout()
-        except NoSuchBranchError, ex:
-            pass
+        except NoSuchBranchError as ex:
+            err = ex
+
+        try:
+            branch = mgr.by_name_prefix(name, remote=True)
+            self.git.checkout(branch.name[len(self.origin_name())+1:])
+            err = None
+        except NoSuchBranchError as ex:
+            if err is None:
+                err = ex
+
+        if err is not None:
+            raise err
 
-        branch = mgr.by_name_prefix(name, remote=True)
-        self.git.checkout(branch.name[len(self.origin_name())+1:])
         return branch
 
     @requires_initialized
@@ -763,6 +830,7 @@ Git config '%s' missing, please fill it in by executing
         # configure remote tracking
         repo.branches[full_name].set_tracking_branch(info.remote_ref)
 
+        return full_name
 
     @requires_initialized
     def pull(self, identifier, remote, name):
