@@ -15,15 +15,20 @@ from gitflow.exceptions import (NotInitialized, GitflowError,
 
 _gitflow = GitFlow()
 
+def check_version_format(version):
+    matcher = _get_version_matcher()
+    if not re.match(matcher + '$', version):
+        raise IllegalVersionFormat(matcher)
+
+def _get_version_matcher():
+    return _gitflow._safe_get('gitflow.release.versionmatcher')
+
 def _get_client():
     return pt.PivotalClient(token=_gitflow._safe_get('gitflow.pt.token'))
 
 def _get_project_id():
     return _gitflow._safe_get('gitflow.pt.projectid')
 
-def _get_version_matcher():
-    return _gitflow._safe_get('gitflow.release.versionmatcher')
-
 def _iter_current_stories():
     client = _get_client()
     pid = _get_project_id()
@@ -32,7 +37,9 @@ def _iter_current_stories():
         return
     for iteration in iterations['iterations']:
         for story in iteration['stories']:
-            yield Story.from_dict(story)
+            s = Story.from_dict(story)
+            if s.is_feature() or s.is_bug():
+                yield s
 
 def _iter_backlog_stories():
     client = _get_client()
@@ -42,21 +49,17 @@ def _iter_backlog_stories():
         return
     for iteration in iterations['iterations']:
         for story in iteration['stories']:
-            yield Story.from_dict(story)
+            s = Story.from_dict(story)
+            if s.is_feature() or s.is_bug():
+                yield s
 
 def _iter_stories():
     return itertools.chain(_iter_current_stories(), _iter_backlog_stories())
 
-def _check_version_format(version):
-    matcher = _get_version_matcher()
-    if not re.match(matcher + '$', version):
-        raise IllegalVersionFormat(matcher)
-
 def list_projects():
     projects = _get_client().projects.all()['projects']
     return [(p['id'], p['name']) for p in projects]
 
-
 class Story(object):
     def __init__(self, story_id, _skip_story_download=False):
         self._project_id = _get_project_id()
@@ -131,6 +134,10 @@ class Story(object):
         assert self.is_feature() or self.is_bug()
         return self.get_state() == 'delivered'
 
+    def is_accepted(self):
+        assert self.is_feature() or self.is_bug()
+        return self.get_state() == 'accepted'
+
     def is_rejected(self):
         assert self.is_feature() or self.is_bug()
         return self.get_state() == 'rejected'
@@ -144,7 +151,7 @@ class Story(object):
 
     def assign_to_release(self, release):
         assert self.is_feature() or self.is_bug()
-        _check_version_format(release)
+        check_version_format(release)
         if self.get_release():
             raise ReleaseAlreadyAssigned('Story already assigned to a release')
         self.add_label('release-' + release)
@@ -181,11 +188,6 @@ class Story(object):
         return self.get_type() == 'bug'
     #--- Bug-specific stuff
 
-    #+++ Chore-specific stuff
-    def is_chore(self):
-        return self.get_type() == 'chore'
-    #--- Chore-specific stuff
-
     def dump(self, highlight_labels=[]):
         story = self._story
         sys.stdout.write(colorize_string(story['name']))
@@ -224,12 +226,17 @@ class Story(object):
 
 class Release(object):
     def __init__(self, version, _skip_story_download=False):
-        _check_version_format(version)
+        check_version_format(version)
         self._version = version
         if _skip_story_download:
             return
         self._current_stories = list(_iter_current_stories())
 
+    def __iter__(self):
+        for story in self._current_stories:
+            if story.is_labeled('release-' + self._version):
+                yield story
+
     def get_version(self):
         assert self._version
         return self._version
@@ -238,34 +245,46 @@ class Release(object):
         for story in self.iter_candidates():
             story.assign_to_release(self._version)
 
-    def try_deliver(self):
+    def try_finish(self):
         err = False
-        for story in self.iter_stories():
+        for story in self:
             if not story.is_labeled('qa+'):
+                # Allow zero-point stories not to be QA'd since they are
+                # by definition refactoring stories.
+                if story.is_feature() and story.get_estimate() == 0:
+                    continue
                 err = True
                 print "    Story not QA'd: " + story.get_url()
+            if story.is_labeled('point me'):
+                err = True
+                print "    Story labeled 'point me': " + story.get_url()
         if err:
-            raise StatusError("There were some stories that were not QA's, operation canceled.")
+            raise StatusError("Pivotal Tracker check did not pass, operation canceled.")
 
-    def deliver(self):
-        print 'Following stories were delivered as of release %s:' \
+    def finish(self):
+        print 'Following stories were delivered for client acceptance as of release %s:' \
               % self._version
-        for story in self.iter_stories():
+        for story in self:
             story.deliver()
             sys.stdout.write('    ')
             story.dump()
 
+    def try_deliver(self):
+        self.try_finish()
+        err = False
+        for story in self:
+            if not story.is_accepted():
+                err = True
+                print('    Story not accepted: ' + story.get_url())
+        if err:
+            raise StatusError("Pivotal Tracker check did not pass, operation canceled.")
+
     def prompt_for_confirmation(self):
         answer = raw_input("Do you wish to start the release? [y/N]: ")
         if answer.lower() == 'y':
             return True
         return False
 
-    def iter_stories(self):
-        for story in self._current_stories:
-            if story.is_labeled('release-' + self._version):
-                yield story
-
     def iter_candidates(self):
         for story in self._current_stories:
             if (story.is_feature() or story.is_bug()) \
@@ -275,7 +294,7 @@ class Release(object):
 
     def dump_stories(self):
         print '%s %s %s %s' % (32 * '-', 'Release', self._version, 33 * '-')
-        for story in self.iter_stories():
+        for story in self:
             story.dump(highlight_labels=['release-' + self._version])
         print 80 * '-' + '\n'
 
@@ -327,8 +346,6 @@ def prompt_user_to_select_story():
     stories = list()
     print Style.DIM + "--------- Stories -----------" + Style.RESET_ALL
     for story in _iter_stories():
-        if story.is_chore():
-            continue
         if story.is_feature() and not story.is_estimated():
             continue
         # You do not start a story if its branch is present or it is
@@ -370,9 +387,12 @@ def prompt_user_to_select_story():
     # We're expecting input to start from 1, so we have to
     # subtract one here to get the list index.
     story = stories[index - 1]
-    slug = prompt_user_to_select_slug(story)
 
-    return [story.get_id(), str(story.get_id()) + '/' + slug]
+    if story.is_rejected():
+        return [story, None]
+
+    slug = prompt_user_to_select_slug(story)
+    return [story, str(story.get_id()) + '/' + slug]
 
 
 def prompt_user_to_select_slug(story):
@@ -478,13 +498,15 @@ def colorize_string(string):
         tmp = tmp.replace(bold, stripped_asterisks)
     return tmp
 
-def get_story_id_from_branch_name(branch_name):
-    match = re.match('^.+/([0-9]+)[/-]?.*$', branch_name)
+
+def get_story_id_from_branch_name(identifier, branch_name):
+    if branch_name.startswith(identifier):
+        name = branch_name[len(identifier)+1:]
+    else:
+        name = branch_name
+    match = re.match('^([0-9]+)[/-]?.*$', name)
     if not match:
-        # Gitflow identifier has already been stripped.
-        match = re.match('^([0-9]+)[/-]?.*$', branch_name)
-        if not match:
-            raise GitflowError('Weird branch name format: %s' % branch_name)
+        raise GitflowError('Weird branch name format: %s' % branch_name)
     return match.groups()[0]
 
 
